# AgentOps Instrumentor for OpenAI Agents SDK

This package provides automatic instrumentation for the OpenAI Agents SDK using AgentOps. It captures detailed telemetry data from agent runs, including spans, metrics, and context information.

## Features

- **Automatic Instrumentation**: Instruments the Agents SDK automatically when imported
- **Comprehensive Span Capture**: Captures all spans from the Agents SDK, including:
  - Agent spans
  - Function spans
  - Generation spans
  - Handoff spans
  - Response spans
  - Custom spans
- **Detailed Metrics**: Collects key metrics such as:
  - Token usage (input/output)
  - Agent execution time
  - Number of agent runs and turns
- **Hybrid Approach**: Combines a custom processor with monkey patching for complete coverage
- **Seamless Integration**: Works with both AgentOps and the Agents SDK's native tracing system

## Installation

The instrumentor is included with the AgentOps package. Simply install AgentOps:

```bash
pip install agentops
```

## Usage

Using the instrumentor is simple - just import it after initializing AgentOps:

```python
# Initialize AgentOps
import agentops
agentops.init(
    instrument_llm_calls=True,
    log_level="DEBUG"
)

# Import the instrumentor - this will automatically instrument the Agents SDK
from opentelemetry.instrumentation.agents import AgentsInstrumentor

# Ensure the instrumentor is registered
instrumentor = AgentsInstrumentor()
instrumentor.instrument()

# Now use the Agents SDK as normal
from agents import Agent, Runner

# Create and run your agents
agent = Agent(name="MyAgent", instructions="You are a helpful assistant.")
result = await Runner.run(agent, "Hello, world!")
```

## Example

See the `agents_instrumentation_example.py` file for a complete example of how to use the instrumentor.

## How It Works

The instrumentor uses two complementary approaches to capture telemetry data:

1. **Custom Processor**: Registers a custom processor with the Agents SDK's tracing system to capture all spans and traces generated by the SDK.

2. **Monkey Patching**: Patches key methods in the Agents SDK to capture additional information that might not be available through the tracing system.

This hybrid approach ensures comprehensive coverage of all agent activities.

## Span Types

The instrumentor captures the following span types:

- **Trace**: The root span representing an entire agent workflow execution
- **Agent**: Represents an agent's execution lifecycle
- **Function**: Represents a tool/function call
- **Generation**: Captures details of model generation
- **Response**: Lightweight span for tracking model response IDs
- **Handoff**: Represents control transfer between agents
- **Custom**: User-defined spans for custom operations

## Metrics

The instrumentor collects the following metrics:

- **Agent Runs**: Number of agent runs
- **Agent Turns**: Number of agent turns
- **Agent Execution Time**: Time taken for agent execution
- **Token Usage**: Number of input and output tokens used

## License

MIT 